// TOP

/*
-------------------------------------------------------------------------------
Copyright 2014 Parallax Inc.

This file is part of the hardware description for the Propeller 1 Design.

The Propeller 1 Design is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by the
Free Software Foundation, either version 3 of the License, or (at your option)
any later version.

The Propeller 1 Design is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
more details.

You should have received a copy of the GNU General Public License along with
the Propeller 1 Design.  If not, see <http://www.gnu.org/licenses/>.
-------------------------------------------------------------------------------
*/

// Based on Rayman's P1V Xyloni release
//https://forums.parallax.com/discussion/174191/p1v-on-xyloni-trion-t8-fpga

module  p1v
(
output  [7:0]       LED_N,              // LEDs, active low 
input               pin_resn,
inout   [31:0]      pin_in
);
//
// Clock generator for Gowin FPGA's
//
wire                clock_125;          // 62.5MHz straight from osc
wire          [7:0] cfg;                // Clock configuration from core
wire                clk_cog;            // Cog clock based on cfg, max 80MHz
wire                clk_pll;            // 2 x clk_cog based on cfg, max 160MHz
wire                n_res;              // Synchronous 50ms reset pulse
wire                clk_i;              // intermediate clock - 160MHZ
                                        //
    Gowin_OSC osc(.oscout(clock_125));  //output oscout                          
gowin_clock gowin_clk   (
    .clock              (clock_125),        // Crystal oscillator on the board
    .cfg                (cfg[7:0]),         // Clock config registers from core
    .res                (n_res),            //
    .clk_pll            (clk_pll),          // clock for cog PLL's
    .clk_cog            (clk_cog)           // clock for instruction execution
);
//
// Reset
//
reset #(
//parameter DELAY_CYCLES = 32'd8_000_000   // 50ms at 160Mhz
) reset (
    .clock          (clock_125),
    .async_res      (pin_resn),
    .res            (n_res)
);
//
// PINS
//
wire[31:0]  pin_out;
wire[31:0]  pin_dir;
genvar n;
generate  
    for (n = 0; n < 32; n++)    begin : generate_pins
        assign pin_in[n]    = pin_dir[n] ? pin_out[n] : 1'bZ;
    end 
endgenerate
//
// LEDs
//
wire          [7:0] cog_led;
assign LED_N = ~cog_led; 
//
// Virtual Propeller
//
    dig core (
    .inp_res        (n_res),
    .cfg            (cfg),
    .clk_cog        (clk_cog),
    .clk_pll        (clk_pll),
    .pin_in         (pin_in),
    .pin_out        (pin_out),
    .pin_dir        (pin_dir),
    .cog_led        (cog_led)
);

endmodule